/*
* This file is part of the acnchess package
* Copyright (C) <2008>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE acccli.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2009/08/12 11:16:40 Alf*/

#include "log.h"
#include "status.h"
#include "accset.h"
#include "acccli.h"

#include <stdlib.h>

static int cli_write_wait (acccli_t *, packet_t *, char *, int, int);

acccli_t *
acccli_new (const char *ip, unsigned short port)
{
  tport_t *tport;

  tport = tport_new (TCP, INADDR_ANY, 0);
  if (tport == NULL)
    {
      acc_err (_("Connect to server [%s:%u error\n"), ip, port);
      return NULL;
    }

  if (tport_connect
      (tport, inet_addr (ip), htons (port),
       g_set->net_conn_timeout * 1000) < 0)
    {
      acc_err (_("Connect to server [%s:%u error\n"), ip, port);
      return NULL;
    }

  return acccli_new_with_tport (tport);
}

acccli_t *
acccli_new_with_tport (tport_t * tport)
{
  acccli_t *cli;
  cli = calloc (1, sizeof (acccli_t));
  if (cli == NULL)
    {
      acc_err (_("Memory error\n"));
      tport_delete (tport);
      return NULL;
    }

  cli->tport = tport;
  cli->status = CONNECT;

  return cli;
}

int
acccli_get_userlist (acccli_t * cli, char *buf, int max)
{
  char rev[1500];
  packet_t pak[1];
  int ret;

  pak->elem = NULL;
  pak->type = ACP_GET_USERLIST;

  ret = cli_write_wait (cli, pak, rev, sizeof rev, g_set->net_read_timeout);
  packet_free (pak);

  if (packet_decode (rev, ret, pak) < 0)
    {
      acc_debug ("get response error\n");
      return -1;
    }

  if (pak->elem && pak->elem->data)
    {
      ret = snprintf (buf, max, "%s", pak->elem->data);
    }
  packet_free (pak);

  return ret;
}

int
acccli_get_userdetail (acccli_t * cli, char *buf, int max)
{
  packet_t pak[1];
  int ret;

  pak->elem = packet_elem_new (ACP_GET_USERDETAIL, 0, NULL);
  pak->type = 1;

  ret = cli_write_wait (cli, pak, buf, max, 3);
  packet_free (pak);

  return ret;
}

void
acccli_delete (acccli_t * cli)
{
  tport_delete (cli->tport);
  free (cli);
}

int
acccli_fight_start (acccli_t * cli)
{
  char buf[1500];
  packet_t pak[1];
  int ret;

  pak->elem = packet_elem_new (ACP_ACTION_REQUEST, 0, NULL);
  pak->type = 1;

  ret = cli_write_wait (cli, pak, buf, sizeof buf, g_set->net_read_timeout);
  packet_free (pak);
  if (packet_decode (buf, ret, pak) < 0)
    {
      return -1;
    }

  return 0;
}

int
acccli_fight_hui (acccli_t * cli)
{
  return 0;
}

int
acccli_fight_quit (acccli_t * cli)
{
  return 0;
}

int
acccli_fight_move (acccli_t * cli, struct xq_move * move)
{
  return 0;
}

struct xq_move *
acccli_get_move (acccli_t * cli)
{
  struct xq_move *move;

  move = NULL;
  return move;
}

static int
cli_write_wait (acccli_t * cli, packet_t * pak, char *buf, int max,
                int timeout)
{
  int siz, ret;

  siz = packet_encode (pak, buf, max);
  if (siz < 0)
    {
      acc_debug ("packet encode error\n");
      return -1;
    }

  ret = tport_write (cli->tport, buf, siz, NULL);
  if (ret < 0)
    {
      acc_debug ("write error\n");
      return -1;
    }

  acc_debug ("write %d[%s] to remote\n", siz, buf);

  ret = tport_read (cli->tport, buf, max, NULL, timeout);
  if (ret > 0)
    {
      acc_debug ("get %d[%s] response from remote\n", ret, buf);
    }

  return ret;
}
