/*
* This file is part of the acnchess package
* Copyright (C) <2008>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE accus.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2009/08/13 10:13:15 Alf*/

#include "log.h"
#include "tport.h"
#include "accxq.h"
#include "acccli.h"
#include "accset.h"
#include "accus.h"

#include <stdlib.h>
#include <string.h>

extern accset_t *g_set;

static void us_load_friends (accus_t *);

static void add_user (gpointer);
static void start_fight (gpointer);

static gpointer us_proxy_thread (gpointer);
static gpointer us_dht_thread (gpointer);

static gboolean usu_on_clicked (GtkWidget * but, GdkEventButton * ev,
                                accus_t * us);
static gboolean usf_on_clicked (GtkWidget * but, GdkEventButton * ev,
                                accus_t * us);

static gboolean us_get_proxy (accus_t *);
static void us_get_proxy_cb (const char *, const char *, void *);

static void us_get_user (accus_t *, const char *, unsigned short);

static int us_add_proxy (accus_t *, const char *, unsigned short);

accus_t *
accus_new ()
{
  accus_t *us;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkWidget *frame, *win, *tree;

  us = calloc (1, sizeof (accus_t));
  if (us == NULL)
    {
      acc_err (_("Memory error\n"));
      return NULL;
    }

  us->widget = gtk_vpaned_new ();

  /*
   * User List
   * */
  frame = gtk_frame_new (_("Users"));
  gtk_paned_add1 (GTK_PANED (us->widget), frame);

  win = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (win),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (win),
                                       GTK_SHADOW_IN);
  gtk_container_add (GTK_CONTAINER (frame), win);

  us->musrs =
    gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
  tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (us->musrs));
  gtk_container_add (GTK_CONTAINER (win), tree);
  us->usel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
  gtk_tree_selection_set_mode (us->usel, GTK_SELECTION_SINGLE);
  g_signal_connect (G_OBJECT (tree), "button-press-event",
                    G_CALLBACK (usu_on_clicked), us);

  /* Nick Name Column */
  renderer = gtk_cell_renderer_text_new ();
  column =
    gtk_tree_view_column_new_with_attributes (_("Nick Name"), renderer,
        "text", 0, NULL);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_column_set_sort_column_id (column, 0);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

  /* Type Column */
  renderer = gtk_cell_renderer_text_new ();
  column =
    gtk_tree_view_column_new_with_attributes (_("Qi Type"), renderer, "text",
        1, NULL);
  gtk_tree_view_column_set_sort_column_id (column, 1);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

  /* Status Column */
  renderer = gtk_cell_renderer_text_new ();
  column =
    gtk_tree_view_column_new_with_attributes (_("Status"), renderer, "text",
        2, NULL);
  gtk_tree_view_column_set_sort_column_id (column, 2);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

  /*
   * Friend List
   * */
  frame = gtk_frame_new (_("Friend List"));
  gtk_paned_add2 (GTK_PANED (us->widget), frame);

  win = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (win),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (win),
                                       GTK_SHADOW_IN);
  gtk_container_add (GTK_CONTAINER (frame), win);

  us->mfriends = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
  tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (us->mfriends));
  gtk_container_add (GTK_CONTAINER (win), tree);

  us->fsel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
  gtk_tree_selection_set_mode (us->fsel, GTK_SELECTION_SINGLE);
  g_signal_connect (G_OBJECT (tree), "button-press-event",
                    G_CALLBACK (usf_on_clicked), us);

  /* IP Column */
  renderer = gtk_cell_renderer_text_new ();
  column =
    gtk_tree_view_column_new_with_attributes (_("IP"), renderer, "text", 0,
        NULL);
  gtk_tree_view_column_set_sort_column_id (column, 0);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

  /* Port Column */
  renderer = gtk_cell_renderer_text_new ();
  column =
    gtk_tree_view_column_new_with_attributes (_("Port"), renderer, "text", 1,
        NULL);
  gtk_tree_view_column_set_sort_column_id (column, 1);
  gtk_tree_view_column_set_resizable (column, TRUE);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

#ifndef _MUDFLAP
  us->dhtthread = g_thread_create (us_dht_thread, us, TRUE, NULL);

  us->proxysrvthread = g_thread_create (us_proxy_thread, us, TRUE, NULL);

  us_load_friends (us);

  us->refuser =
    g_timeout_add (100, (gboolean (*)(gpointer)) us_get_proxy, us);
#endif

  return us;
}

void
accus_delete (accus_t * us)
{
  struct proxy_node *pn;

  if (us->proxysrvthread != NULL)
    {
      if (us->proxysrv != NULL)
        {
          proxysrv_stop (us->proxysrv);
          g_thread_join (us->proxysrvthread);
          proxysrv_delete (us->proxysrv);
        }
    }

  if (us->refuser > 0)
    {
      g_source_remove (us->refuser);
    }
  if (us->dht)
    {
      dht_delete (us->dht);
    }

  while ((pn = LIST_FIRST (us->proxy_list)) != NULL)
    {
      LIST_REMOVE (pn, entries);
      free (pn);
    }

  free (us);
}

static void
us_load_friends (accus_t * us)
{
  char line[0x100], *p;
  GtkTreeIter itr[1];
  size_t len;

  gtk_list_store_clear (us->mfriends);

  while (NULL != NULL)
    {
      p = strchr (line, ':');
      if (p != NULL)
        {
          *p = '\0';
          p++;
          len = strlen (p);
          while (p[len - 1] < '0' || p[len - 1] > '9')
            {
              len--;
            }
          p[len] = '\0';
          gtk_list_store_append (us->mfriends, itr);
          gtk_list_store_set (us->mfriends, itr, 0, line, 1, p, -1);
          acc_debug (_("add a server [%s:%d]\n"), line, atoi (p));
        }
    }
}

static gboolean
us_get_proxy (accus_t * us)
{
  if (us->dht)
    {
      dht_pub_keyword (us->dht, ACC_PROXY_SERVER_TOKEN, g_set->net_tcp);
      dht_get_keyword (us->dht, ACC_PROXY_SERVER_TOKEN, us_get_proxy_cb, us);
    }

  us->refuser =
    g_timeout_add (g_set->net_ref_user * 1000,
                   (gboolean (*)(gpointer)) us_get_proxy, us);

  return FALSE;
}

static int dht_fd;
static ssize_t dht_read (void *, char *, size_t, struct sockaddr_in *, int);
static ssize_t dht_write (void *, const char *, size_t, struct sockaddr_in *);

static gpointer
us_dht_thread (gpointer arg)
{
  accus_t *us;
  char *rfile, *real;
  struct sockaddr_in addr[1];
  dhtio_t dhtio[1] =
  {
    {
      &dht_fd,
      dht_read,
      dht_write
    }
  };

  us = (accus_t *) arg;

  rfile = acc_realpath ("~/.acnchess/dht.ini");
  if (rfile != NULL)
    {
      real = g_locale_from_utf8 (rfile, -1, NULL, NULL, NULL);
      free (rfile);
    }
  else
    {
      real = NULL;
    }

  dht_fd = socket (AF_INET, SOCK_DGRAM, 0);

  addr->sin_family = AF_INET;
  addr->sin_addr.s_addr = inet_addr ("0.0.0.0");
  addr->sin_port = htons (6681);
  memset (addr->sin_zero, 0, 8);

  bind (dht_fd, (struct sockaddr *) addr, sizeof (struct sockaddr));

  if (real != NULL)
    {
      us->dht = dht_new (real, g_set->net_udp, dhtio);
      g_free (real);
    }
  else
    {
      us->dht = dht_new ("dht.ini", g_set->net_udp, dhtio);
    }

  return us->dht;
}


static void
us_get_proxy_cb (const char *key, const char *str, void *arg)
{
  accus_t *us;
  char ip[20], port[6];
  GtkTreeIter itr[1];

  us = (accus_t *) arg;

  snprintf (ip, sizeof ip, "%s", inet_ntoa (*(struct in_addr *) str));
  snprintf (port, sizeof port, "%d", ntohs (*(unsigned short *) (str + 4)));
  if (us_add_proxy (us, ip, atoi (port)) == 0)
    {
      return;
    }

  gtk_list_store_append (us->mfriends, itr);
  gtk_list_store_set (us->mfriends, itr, 0, ip, 1, port, -1);

  acc_debug ("Find [%s], get ip and port: %s:%d\n", key, ip, atoi (port));
  us_get_user (us, ip, atoi (port));
}

static void
us_get_user (accus_t * us, const char *ip, unsigned short port)
{
  acccli_t *cli;
  char buf[1500], **users, **userdv;
  int ret, i;
  GtkTreeIter itr[1];

#ifdef _DEBUG
  cli = acccli_new ("127.0.0.1", port);
#else
  cli = acccli_new (ip, port);
#endif
  if (cli == NULL)
    {
      acc_err (_("Memory error\n"));
      return;
    }

  ret = acccli_get_userlist (cli, buf, sizeof buf - 1);
  if (ret <= 0)
    {
      acccli_delete (cli);
      return;
    }

  acc_debug ("get %d userlist response\n", ret);

  buf[ret] = '\0';
  users = g_strsplit (buf, "\n", 0);
  for (i = 0; users[i] != NULL; i++)
    {
      userdv = g_strsplit (users[i], ":", 3);
      if (userdv == NULL)
        {
          continue;
        }

      if (g_strv_length (userdv) < 3)
        {
          g_strfreev (userdv);
          continue;
        }

      gtk_list_store_append (us->musrs, itr);
      gtk_list_store_set (us->musrs, itr, 0, userdv[0], 1, userdv[1], 2,
                          userdv[2], -1);
      g_strfreev (userdv);
    }
  g_strfreev (users);

  acccli_delete (cli);
}

static int
us_add_proxy (accus_t * us, const char *ip, unsigned short port)
{
  struct proxy_node *pn;

  LIST_FOREACH (pn, us->proxy_list, entries)
  {
    if (inet_addr (ip) == pn->ip && htons (port) == pn->port)
      {
        return 0;
      }
  }

  pn = calloc (1, sizeof (struct proxy_node));
  if (pn == NULL)
    {
      acc_err (_("Memory error\n"));
      return 0;
    }

  pn->ip = inet_addr (ip);
  pn->port = htons (port);
  LIST_INSERT_HEAD (us->proxy_list, pn, entries);
  return 1;
}

static gpointer
us_proxy_thread (gpointer arg)
{
  accus_t *us;

  us = (accus_t *) arg;

  us->proxysrv = proxysrv_new (10, g_set->net_tcp);

  if (us->proxysrv != NULL)
    {
      proxysrv_start (us->proxysrv);
    }

  return us;
}

static gboolean
usu_on_clicked (GtkWidget * but, GdkEventButton * ev, accus_t * us)
{
  GtkTreeModel *model;
  GtkWidget *menu;
  gboolean has;

  if (ev->button != 3)
    return FALSE;

  has = FALSE;
  model = GTK_TREE_MODEL (us->musrs);
  if (gtk_tree_selection_get_selected (us->usel, &model, us->uitr))
    {
      has = TRUE;
    }

  acc_debug ("right clicked at %p: %s.\n", but, has ? "has" : "");

  menu = gtk_menu_new ();
  gtk_menu_add_item (menu, _("Add"), add_user, us);
  if (has)
    {
      gtk_menu_add_item (menu, _("Fight"), start_fight, us);
//      gtk_menu_add_item (menu, _ ("Del"), NULL, us);
//      gtk_menu_add_item (menu, _ ("Edit"), NULL, us);
    }
  gtk_widget_show_all (menu);

  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, ev->button,
                  ev->time);
  return TRUE;
}

static gboolean
usf_on_clicked (GtkWidget * but, GdkEventButton * ev, accus_t * us)
{
  GtkTreeModel *model;
  gboolean has;

  if (ev->button != 3)
    return FALSE;

  has = FALSE;
  model = GTK_TREE_MODEL (us->musrs);
  if (gtk_tree_selection_get_selected (us->usel, &model, us->fitr))
    {
      has = TRUE;
    }

  acc_info ("right clicked at %p: %s.\n", but, has ? "has" : "");

  return TRUE;
}

static void
add_user (gpointer arg)
{
}

static void
start_fight (gpointer arg)
{
  /*
  char ip[0x10];
  unsigned short port;
  acccli_t *cli;
  session_t *ses;
  accus_t *us;
  us = (accus_t *) arg;

  cli = acccli_new (ip, port);
  ses = session_new_with_cli (ZGXIANGQI, cli);
  pannode_new (g_xq, ses);*/
}

static ssize_t
dht_read (void *p, char *buf, size_t size, struct sockaddr_in *sa, int tim)
{
  int fd;
  ssize_t ret;
  socklen_t siz;
  fd_set set[1];
  struct timeval tv[1];

  fd = * (int *) p;

  FD_ZERO (set);
  FD_SET (fd, set);
  tv->tv_sec = tim;
  tv->tv_usec = 0;
  if (select (fd + 1, set, NULL, NULL, tv) <= 0
      || !FD_ISSET (fd, set))
    {
      return -1;
    }

  siz = sizeof (struct sockaddr);
  ret = recvfrom (fd, buf, size, 0, (struct sockaddr *) sa, &siz);

  return ret;
}

static ssize_t
dht_write (void *p, const char *buf, size_t size, struct sockaddr_in *sa)
{
  int fd;
  ssize_t ret;
  socklen_t siz;

  fd = * (int *) p;
  siz = sizeof (struct sockaddr);
  ret = sendto (fd, buf, size, 0, (struct sockaddr *) sa, siz);

  return ret;
}
