/*
* This file is part of the acnchess package
* Copyright (C) <2008>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE accxq.h
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2009/07/30 13:40:44 Alf*/

#ifndef _ACC_XIANGQI_H_
#define _ACC_XIANGQI_H_

#include "acccli.h"
#include "acnkern.h"
#include "session.h"
#include "qipu.h"

#include <gtk/gtk.h>

typedef struct accxq_s accxq_t;

enum
{
  PAN_STATE_INIT,
  PAN_STATE_NOT_TURN,
  PAN_STATE_TURN,
  PAN_STATE_TURN_SELECTED,
  PAN_STATE_GAME_OVER,
  PAN_STATE_QIPU,
};

struct pan_node
{
  /*
   * the name and the description
   */
  char *name;
  char *description;

  /*
   * the xq struct
   * */
  accxq_t * accxq;

  GtkWidget *parent;
  GtkWidget *widget;
  GtkWidget *drawarea;

  gint width, height;
  gint fromx, fromy, tox, toy, perx, pery, qix, qiy;

  gint motionx, motiony;

  gint selx, sely;

  int state;

  /* the begin fen */
  fen_t fen[1];

  /* the current fen and moves */
  fen_t cur_fen[1];
  GList *cur_moves;

  /* the total move queues */
  struct xq_move moves[1024];
  gint move_count;

  /* the curren pan
   * 'cur_fen' can tranlate to 'cur_pan' through 'cur_moves' */
  char cur_pan[10][9];

  session_t *session;

  GtkWidget *bbaiqi, *bstart, *bstop, *bhui;

  gint other_type;

  GtkWidget *wconf;
  gint conf_id;
  GtkWidget *conf[8];

  qipu_t *qipu;
  char *qipu_file;
  GtkListStore *qipu_list;
  GtkTreeSelection *qipu_sel;
  GtkLabel *qipu_desc;

  GThread *parterthread;
};

struct accxq_s
{
  /*
   * UI Widgets
   * */
  GtkWidget *widget;

  /*
   * Every Pan
   * */
  GtkWidget *pantab;
  GList * panlist;

  GtkWidget *bload, *bsave, *bprev, *bnext;
};

#define accxq_widget(x)         ((x)->widget)

accxq_t *accxq_new ();

void accxq_delete (accxq_t *);

struct pan_node *pannode_new (accxq_t *, session_t *);

void pannode_delete (struct pan_node *);

extern accxq_t *g_xq;

#endif
