/*
* This file is part of the acnchess package
* Copyright (C) <2008>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE zgxq.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2009/08/01 00:40:18 Alf*/

#include "acnkern.h"
#include "qipu.h"
#include "log.h"
#include "zgxq.h"

#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <time.h>

#define BASEVALUE_BING          100
#define BASEVALUE_SHI           250
#define BASEVALUE_XIANG         250
#define BASEVALUE_JU            500
#define BASEVALUE_MA            350
#define BASEVALUE_PAO           350
#define BASEVALUE_SHUAI         10000

#define FLEXIBILITY_BING        15
#define FLEXIBILITY_SHI         1
#define FLEXIBILITY_XIANG       1
#define FLEXIBILITY_JU          6
#define FLEXIBILITY_MA          12
#define FLEXIBILITY_PAO         6
#define FLEXIBILITY_SHUAI       0

const int rbing[10][9] =
{
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {90, 90, 110, 120, 120, 120, 110, 90, 90,},
  {90, 90, 110, 120, 120, 120, 110, 90, 90,},
  {70, 90, 110, 110, 110, 110, 110, 90, 70,},
  {70, 70, 70, 70, 70, 70, 70, 70, 70,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
};

const int bbing[10][9] =
{
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {70, 70, 70, 70, 70, 70, 70, 70, 70,},
  {70, 90, 110, 110, 110, 110, 110, 90, 70,},
  {90, 90, 110, 120, 120, 120, 110, 90, 90,},
  {90, 90, 110, 120, 120, 120, 110, 90, 90,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
};

#define getbingvalue(x,y,pan)         (pan)[y][x] == R_BING? rbing[y][x]: (pan)[y][x] == B_BING? bbing[y][x]: 0

typedef struct
{
  int x;
  int y;
} zgxq_pos_t;

enum
{ AH_EXACT, AH_LOWER, AH_UPPER };

typedef struct
{
  unsigned long long cksum;
  int type;
  short depth;
  short eval;
} zgxq_hash_t;

static void zgxq_table_cal_init (acnkern_t *, char[10][9]);

static void zgxq_table_make_move (acnkern_t *, struct xq_move *,
char[10][9]);

static void zgxq_table_unmake_move (acnkern_t *, struct xq_move *, int,
char[10][9]);

static int zgxq_table_find (acnkern_t *, int, int, int, int);

static void zgxq_table_store (acnkern_t *, int, short, short, int);

static void zgxq_table_init (acnkern_t *);

struct acnkern_s
{
  int maxdepth;

  char pan[10][9];

  struct xq_move *bestmove;

  /* Generator */
  struct xq_move list[8][100];
  int count;

  /* Eveluator */
  int basevalue[15];
  int flexvalue[15];
  short attackpos[10][9];
  char guardpos[10][9];
  char flexibilitypos[10][9];
  int chessvalue[10][9];
  int poscount;
  zgxq_pos_t relatepos[20];

  /* Hash Table */
  unsigned int hashkey32[15][10][9];
  unsigned long long hashkey64[15][10][9];
  zgxq_hash_t head[2][1024 * 1024];
  unsigned int key32;
  unsigned long long key64;

  int callback_count;
  int eveluate_count;

  acnkern_callback callback;
  acnkern_callback_arg callbackarg;

  int quit;
};

static int g_count;

static void zgxq_move_shuai (acnkern_t *, char[10][9], int, int, int);

static void zgxq_move_rshi (acnkern_t *, char[10][9], int, int, int);

static void zgxq_move_bshi (acnkern_t *, char[10][9], int, int, int);

static void zgxq_move_xiang (acnkern_t *, char[10][9], int, int, int);

static void zgxq_move_ma (acnkern_t *, char[10][9], int, int, int);

static void zgxq_move_ju (acnkern_t *, char[10][9], int, int, int);

static void zgxq_move_pao (acnkern_t *, char[10][9], int, int, int);

static void zgxq_move_rbing (acnkern_t *, char[10][9], int, int, int);

static void zgxq_move_bbing (acnkern_t *, char[10][9], int, int, int);

static char acnkern_make_move (acnkern_t *, struct xq_move *);

static void acnkern_unmake_move (acnkern_t *, struct xq_move *, char);

static int acnkern_is_over (acnkern_t *, char[10][9], int);

static int acnkern_get_relate (acnkern_t *, char[10][9], int, int);

static int zgxq_eveluate (acnkern_t *, char[10][9], int);

static int
zgxq_gen_add_move (acnkern_t * gen, int fx, int fy, int tx, int ty,
int level)
{
  gen->list[level][gen->count].fromx = fx;
  gen->list[level][gen->count].fromy = fy;
  gen->list[level][gen->count].tox = tx;
  gen->list[level][gen->count].toy = ty;
  gen->count++;
//  acc_debug ("add move: level: %d from[%d:%d] to [%d:%d]\n", level, fx, fy, tx, ty);
  return gen->count;
}

static int
zgxq_gen_create_possible (acnkern_t * gen, char pan[10][9], int level,
int isred)
{
  int id, i, j;

  gen->count = 0;
  for (j = 0; j < 9; ++j)
    {
      for (i = 0; i < 10; ++i)
        {
          if (pan[i][j] != 0)
            {
              id = pan[i][j];
              if (!isred && IS_RED (id))
                continue;
              if (isred && IS_BLACK (id))
                continue;

              switch (id)
                {
                case R_SHUAI:
                case B_SHUAI:
                  zgxq_move_shuai (gen, pan, i, j, level);
                  break;

                case R_SHI:
                  zgxq_move_rshi (gen, pan, i, j, level);
                  break;

                case B_SHI:
                  zgxq_move_bshi (gen, pan, i, j, level);
                  break;

                case R_XIANG:
                case B_XIANG:
                  zgxq_move_xiang (gen, pan, i, j, level);
                  break;

                case R_MA:
                case B_MA:
                  zgxq_move_ma (gen, pan, i, j, level);
                  break;

                case R_JU:
                case B_JU:
                  zgxq_move_ju (gen, pan, i, j, level);
                  break;

                case R_BING:
                  zgxq_move_rbing (gen, pan, i, j, level);
                  break;

                case B_BING:
                  zgxq_move_bbing (gen, pan, i, j, level);
                  break;

                case R_PAO:
                case B_PAO:
                  zgxq_move_pao (gen, pan, i, j, level);
                  break;

                default:
                  break;
                }
            }
        }
    }

  return gen->count;
}

static void
zgxq_move_shuai (acnkern_t * gen, char pan[10][9], int i, int j,
int level)
{
  int x, y;

  for (y = 0; y < 3; ++y)
    {
      for (x = 3; x < 6; ++x)
        {
          if (zgxq_move_is_valid (pan, j, i, x, y))
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
        }
    }

  for (y = 7; y < 10; y++)
    {
      for (x = 3; x < 6; ++x)
        {
          if (zgxq_move_is_valid (pan, j, i, x, y))
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
        }
    }
}

static void
zgxq_move_rshi (acnkern_t * gen, char pan[10][9], int i, int j, int level)
{
  int x, y;

  for (y = 7; y < 10; ++y)
    {
      for (x = 3; x < 6; ++x)
        {
          if (zgxq_move_is_valid (pan, j, i, x, y))
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
        }
    }
}

static void
zgxq_move_bshi (acnkern_t * gen, char pan[10][9], int i, int j, int level)
{
  int x, y;

  for (y = 0; y < 3; ++y)
    {
      for (x = 3; x < 6; ++x)
        {
          if (zgxq_move_is_valid (pan, j, i, x, y))
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
        }
    }
}

static void
zgxq_move_xiang (acnkern_t * gen, char pan[10][9], int i, int j,
int level)
{
  int x, y;

  x = j + 2;
  y = i + 2;
  if (x < 9 && y < 10 && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j + 2;
  y = i - 2;
  if (x < 9 && y >= 0 && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j - 2;
  y = i + 2;
  if (x >= 0 && y < 10 && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j - 2;
  y = i - 2;
  if (x >= 0 && y >= 0 && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }
}

static void
zgxq_move_ma (acnkern_t * gen, char pan[10][9], int i, int j, int level)
{
  int x, y;

  x = j + 2;
  y = i + 1;
  if ((x < 9 && y < 10) && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j + 2;
  y = i - 1;
  if ((x < 9 && y >= 0) && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j - 2;
  y = i + 1;
  if ((x >= 0 && y < 10) && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j - 2;
  y = i - 1;
  if ((x >= 0 && y >= 0) && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j + 1;
  y = i + 2;
  if ((x < 9 && y < 10) && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j - 1;
  y = i + 2;
  if ((x >= 0 && y < 10) && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j + 1;
  y = i - 2;
  if ((x < 9 && y >= 0) && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  x = j - 1;
  y = i - 2;
  if ((x >= 0 && y >= 0) && zgxq_move_is_valid (pan, j, i, x, y))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }
}

static void
zgxq_move_ju (acnkern_t * gen, char pan[10][9], int i, int j, int level)
{
  int x, y, id;

  id = pan[i][j];

  for (x = j + 1, y = i; x < 9; ++x)
    {
      if (pan[y][x] == 0)
        {
          zgxq_gen_add_move (gen, j, i, x, y, level);
        }
      else
        {
          if (!IS_SAME (id, pan[y][x]))
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
          break;
        }
    }

  for (x = j - 1, y = i; x >= 0; --x)
    {
      if (pan[y][x] == 0)
        {
          zgxq_gen_add_move (gen, j, i, x, y, level);
        }
      else
        {
          if (!IS_SAME (id, pan[y][x]))
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
          break;
        }
    }

  for (x = j, y = i + 1; y < 10; ++y)
    {
      if (pan[y][x] == 0)
        {
          zgxq_gen_add_move (gen, j, i, x, y, level);
        }
      else
        {
          if (!IS_SAME (id, pan[y][x]))
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
          break;
        }
    }

  for (x = j, y = i - 1; y >= 0; --y)
    {
      if (pan[y][x] == 0)
        {
          zgxq_gen_add_move (gen, j, i, x, y, level);
        }
      else
        {
          if (!IS_SAME (id, pan[y][x]))
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
          break;
        }
    }
}

static void
zgxq_move_pao (acnkern_t * gen, char pan[10][9], int i, int j, int level)
{
  int x, y, flag, id;

  id = pan[i][j];

  for (x = j + 1, y = i, flag = 0; x < 9; ++x)
    {
      if (pan[y][x] == 0)
        {
          if (!flag)
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
        }
      else
        {
          if (!flag)
            {
              flag = 1;
            }
          else
            {
              if (!IS_SAME (id, pan[y][x]))
                {
                  zgxq_gen_add_move (gen, j, i, x, y, level);
                }
              break;
            }
        }
    }

  for (x = j - 1, y = i, flag = 0; x >= 0; --x)
    {
      if (pan[y][x] == 0)
        {
          if (!flag)
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
        }
      else
        {
          if (!flag)
            {
              flag = 1;
            }
          else
            {
              if (!IS_SAME (id, pan[y][x]))
                {
                  zgxq_gen_add_move (gen, j, i, x, y, level);
                }
              break;
            }
        }
    }

  for (x = j, y = i + 1, flag = 0; y < 10; ++y)
    {
      if (pan[y][x] == 0)
        {
          if (!flag)
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
        }
      else
        {
          if (!flag)
            {
              flag = 1;
            }
          else
            {
              if (!IS_SAME (id, pan[y][x]))
                {
                  zgxq_gen_add_move (gen, j, i, x, y, level);
                }
              break;
            }
        }
    }

  for (x = j, y = i - 1, flag = 0; y >= 0; --y)
    {
      if (pan[y][x] == 0)
        {
          if (!flag)
            {
              zgxq_gen_add_move (gen, j, i, x, y, level);
            }
        }
      else
        {
          if (!flag)
            {
              flag = 1;
            }
          else
            {
              if (!IS_SAME (id, pan[y][x]))
                {
                  zgxq_gen_add_move (gen, j, i, x, y, level);
                }
              break;
            }
        }
    }
}

static void
zgxq_move_rbing (acnkern_t * gen, char pan[10][9], int i, int j,
int level)
{
  int x, y, id;

  id = pan[i][j];

  y = i - 1;
  x = j;
  if (y >= 0 && !IS_SAME (id, pan[y][x]))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }

  if (i < 5)
    {
      y = i;
      x = j + 1;
      if (x < 9 && !IS_SAME (id, pan[y][x]))
        {
          zgxq_gen_add_move (gen, j, i, x, y, level);
        }
      x = j - 1;
      if (x >= 0 && !IS_SAME (id, pan[y][x]))
        {
          zgxq_gen_add_move (gen, j, i, x, y, level);
        }
    }
}

static void
zgxq_move_bbing (acnkern_t * gen, char pan[10][9], int i, int j,
int level)
{
  int x, y, id;

  id = pan[i][j];
  y = i + 1;
  x = j;
  if (y < 10 && !IS_SAME (id, pan[y][x]))
    {
      zgxq_gen_add_move (gen, j, i, x, y, level);
    }
  if (i > 4)
    {
      y = i;
      x = j + 1;
      if (x < 9 && !IS_SAME (id, pan[y][x]))
        {
          zgxq_gen_add_move (gen, j, i, x, y, level);
        }
      x = j - 1;
      if (x >= 0 && !IS_SAME (id, pan[y][x]))
        {
          zgxq_gen_add_move (gen, j, i, x, y, level);
        }
    }
}

#ifdef _DEBUG
static int
acnkern_principal_variation (acnkern_t * search, int depth, int alpha,
int beta)
{
  int score, count, i, best;
  char type;

  i = acnkern_is_over (search, search->pan, depth);
  if (i != 0)
    {
      return i;
    }

  /*
     score = zgxq_table_find (search, alpha, beta, depth,
     (search->maxdepth - depth) % 2);
     if (score != 66666)
     {
     return score;
     } */

  if (depth <= 0)
    {
      score =
      zgxq_eveluate (search, search->pan, (search->maxdepth - depth) % 2);
      zgxq_table_store (search, AH_EXACT, score, depth,
      (search->maxdepth - depth) % 2);
      return score;
    }

  count =
  zgxq_gen_create_possible (search, search->pan, depth,
  (search->maxdepth - depth) % 2);

  zgxq_table_make_move (search, search->list[depth], search->pan);
  type = acnkern_make_move (search, search->list[depth]);
  best = -acnkern_principal_variation (search, depth - 1, -beta, -alpha);
  zgxq_table_unmake_move (search, search->list[depth], type, search->pan);
  acnkern_unmake_move (search, search->list[depth], type);

  if (search->maxdepth == depth)
    {
      search->bestmove = search->list[depth];
    }

  for (i = 1; i < count; ++i)
    {
      if (best < beta)
        {
          if (best > alpha)
            {
              alpha = best;
            }
          zgxq_table_make_move (search, search->list[depth] + i, search->pan);
          type = acnkern_make_move (search, search->list[depth] + i);
          score =
          -acnkern_principal_variation (search, depth - 1, -alpha - 1,
          -alpha);

          if (score > alpha && score < beta)
            {
              best =
              -acnkern_principal_variation (search, depth - 1, -beta,
              -score);
              if (search->maxdepth == depth)
                {
                  search->bestmove = search->list[depth] + i;
                }
              /*  zgxq_table_store (search, AH_UPPER, best, depth,
                 (search->maxdepth - depth) % 2); */
            }
          else if (score > best)
            {
              best = score;
              if (search->maxdepth == depth)
                {
                  search->bestmove = search->list[depth] + i;
                }
              /*
                 zgxq_table_store (search, AH_LOWER, best, depth,
                 (search->maxdepth - depth) % 2); */
            }
          zgxq_table_unmake_move (search,
          search->list[depth], type, search->pan);
          acnkern_unmake_move (search, search->list[depth] + i, type);
        }
    }

  return best;
}
#endif

static int
acnkern_f_alpha_beta (acnkern_t * search, int depth, int alpha,
int beta)
{
  int score, count, i, is_exact;
  char type;

  if (search->quit == 1)
    {
      return 0;
    }

  i = acnkern_is_over (search, search->pan, depth);
  if (i != 0)
    {
      return i;
    }

  score =
  zgxq_table_find (search, alpha, beta, depth,
  (search->maxdepth - depth) % 2);
  if (score != 66666)
    {
      return score;
    }

  if (depth <= 0)
    {
      score =
      zgxq_eveluate (search, search->pan, (search->maxdepth - depth) % 2);
      zgxq_table_store (search, AH_EXACT, score, depth,
      (search->maxdepth - depth) % 2);
      return score;
    }

  count =
  zgxq_gen_create_possible (search, search->pan, depth,
  (search->maxdepth - depth) % 2);

  for (is_exact = i = 0; i < count; ++i)
    {
      zgxq_table_make_move (search, search->list[depth] + i, search->pan);
      type = acnkern_make_move (search, search->list[depth] + i);
      score = 0 - acnkern_f_alpha_beta (search, depth - 1, -beta, -alpha);
      zgxq_table_unmake_move (search,
      search->list[depth] + i, type, search->pan);
      acnkern_unmake_move (search, search->list[depth] + i, type);

      if (score >= beta)
        {
          zgxq_table_store (search, AH_LOWER, score, depth,
          (search->maxdepth - depth) % 2);
          return score;
        }
      if (score > alpha)
        {
          alpha = score;
          is_exact = 1;
          if (search->maxdepth == depth)
            {
              search->bestmove = search->list[depth] + i;
            }
        }
    }

  if (is_exact)
    {
      zgxq_table_store (search, AH_EXACT, alpha, depth,
      (search->maxdepth - depth) % 2);
    }
  else
    {
      zgxq_table_store (search, AH_UPPER, alpha, depth,
      (search->maxdepth - depth) % 2);
    }

  return alpha;
}

int
acnkern_nega_max (acnkern_t * search, int depth)
{
  int current, score, count, i;
  char type;

  i = acnkern_is_over (search, search->pan, depth);
  if (i != 0)
    {
      return i;
    }

  if (depth <= 0)
    {
      score = zgxq_eveluate (search, search->pan,
      (search->maxdepth - depth) % 2);
      return score;
    }

  current = -20000;
  count =
  zgxq_gen_create_possible (search, search->pan, depth,
  (search->maxdepth - depth) % 2);
  for (i = 0; i < count; ++i)
    {
      type = acnkern_make_move (search, search->list[depth] + i);
      score = -acnkern_nega_max (search, depth - 1);
      acnkern_unmake_move (search, search->list[depth] + i, type);
      if (score > current)
        {
          current = score;
          if (search->maxdepth == depth)
            {
              search->bestmove = search->list[depth] + i;
            }
        }
    }

  return current;
}

acnkern_t *
acnkern_new (acnkern_callback cb, acnkern_callback_arg arg)
{
  acnkern_t *search;

  search = calloc (1, sizeof (acnkern_t));
  if (search == NULL)
    {
      fprintf (stderr, _("Memory error\n"));
      return NULL;
    }

  search->basevalue[R_SHUAI] = BASEVALUE_SHUAI;
  search->basevalue[R_JU] = BASEVALUE_JU;
  search->basevalue[R_MA] = BASEVALUE_MA;
  search->basevalue[R_SHI] = BASEVALUE_SHI;
  search->basevalue[R_XIANG] = BASEVALUE_XIANG;
  search->basevalue[R_PAO] = BASEVALUE_PAO;
  search->basevalue[R_BING] = BASEVALUE_BING;
  search->basevalue[B_SHUAI] = BASEVALUE_SHUAI;
  search->basevalue[B_JU] = BASEVALUE_JU;
  search->basevalue[B_MA] = BASEVALUE_MA;
  search->basevalue[B_SHI] = BASEVALUE_SHI;
  search->basevalue[B_XIANG] = BASEVALUE_XIANG;
  search->basevalue[B_PAO] = BASEVALUE_PAO;
  search->basevalue[B_BING] = BASEVALUE_BING;

  search->flexvalue[R_SHUAI] = FLEXIBILITY_SHUAI;
  search->flexvalue[R_JU] = FLEXIBILITY_JU;
  search->flexvalue[R_MA] = FLEXIBILITY_MA;
  search->flexvalue[R_SHI] = FLEXIBILITY_SHI;
  search->flexvalue[R_XIANG] = FLEXIBILITY_XIANG;
  search->flexvalue[R_PAO] = FLEXIBILITY_PAO;
  search->flexvalue[R_BING] = FLEXIBILITY_BING;
  search->flexvalue[B_SHUAI] = FLEXIBILITY_SHUAI;
  search->flexvalue[B_JU] = FLEXIBILITY_JU;
  search->flexvalue[B_MA] = FLEXIBILITY_MA;
  search->flexvalue[B_SHI] = FLEXIBILITY_SHI;
  search->flexvalue[B_XIANG] = FLEXIBILITY_XIANG;
  search->flexvalue[B_PAO] = FLEXIBILITY_PAO;
  search->flexvalue[B_BING] = FLEXIBILITY_BING;

  zgxq_table_init (search);

  return search;
}

void
acnkern_set_depth (acnkern_t * search, int depth)
{
  acc_debug ("set search depth to '%d'\n", depth);
  search->maxdepth = depth;
}

void 
acnkern_set_callback_count (acnkern_t * search, int count)
{
  acc_debug ("set callback count to '%d'\n", count);
  search->callback_count = count;
}

void
acnkern_delete (acnkern_t * search)
{
  free (search);
}

int 
acnkern_set_pan (acnkern_t *kern, char pan[10][9])
{
  memcpy (kern->pan, pan, 10 * 9 * sizeof (char));
  return 0;
}

int 
acnkern_set_move (acnkern_t *kern, struct xq_move *move)
{
  pan_move (kern->pan, move->fromx, move->fromy, move->tox, move->toy);
  return 0;
}

int 
acnkern_set_banmove (acnkern_t *kern, struct xq_move *move)
{
  return 0;
}

//#define FALPHA
struct xq_move *
acnkern_go (acnkern_t * search, int milsecs)
{
  zgxq_table_cal_init (search, search->pan);

#ifdef _DEBUG
  acc_debug ("Used Principal Variation\n");
  g_count = 0;
  acnkern_principal_variation (search, search->maxdepth, -20000, 20000);
  acc_debug ("eveluate %d times, return: %d:%d->%d:%d\n", g_count,
  search->bestmove->fromx, search->bestmove->fromy,
  search->bestmove->tox, search->bestmove->toy);

  acc_debug ("Used Fail Alpha Beta\n");
  g_count = 0;
  acnkern_f_alpha_beta (search, search->maxdepth, -20000, 20000);
  acc_debug ("eveluate %d times, return: %d:%d->%d:%d\n", g_count,
  search->bestmove->fromx, search->bestmove->fromy,
  search->bestmove->tox, search->bestmove->toy);

  if (search->maxdepth > 3)
    {
      return search->bestmove;
    }

  acc_debug ("Used Nega Max\n");
  g_count = 0;
  acnkern_nega_max (search, search->maxdepth);
  acc_debug ("eveluate %d times, return: %d:%d->%d:%d\n", g_count,
  search->bestmove->fromx, search->bestmove->fromy,
  search->bestmove->tox, search->bestmove->toy);
#else
  acnkern_f_alpha_beta (search, search->maxdepth, -20000, 20000);
#endif

  return search->bestmove;
}

static char
acnkern_make_move (acnkern_t * search, struct xq_move * move)
{
  char id;

  id = search->pan[move->toy][move->tox];
  pan_move (search->pan, move->fromx, move->fromy, move->tox, move->toy);
  return id;
}

static void
acnkern_unmake_move (acnkern_t * search, struct xq_move * move, char id)
{
  search->pan[move->fromy][move->fromx] = search->pan[move->toy][move->tox];
  search->pan[move->toy][move->tox] = id;
}

static int
acnkern_is_over (acnkern_t * search, char pan[10][9], int depth)
{
  int i, j, redlive = 0, blacklive = 0;

  for (i = 7; i < 10; ++i)
    {
      for (j = 3; j < 6; ++j)
        {
          if (pan[i][j] == B_SHUAI)
            {
              blacklive = 1;
            }
          if (pan[i][j] == R_SHUAI)
            {
              redlive = 1;
            }
        }
    }

  for (i = 0; i < 3; ++i)
    {
      for (j = 3; j < 6; ++j)
        {
          if (pan[i][j] == B_SHUAI)
            {
              blacklive = 1;
            }
          if (pan[i][j] == R_SHUAI)
            {
              redlive = 1;
            }
        }
    }

  i = (search->maxdepth - depth + 1) % 2;
  if (!redlive)
    {
      if (i)
        {
          return 19990 + depth;
        }
      else
        {
          return -19990 - depth;
        }
    }
  if (!blacklive)
    {
      if (i)
        {
          return -19990 - depth;
        }
      else
        {
          return 19990 + depth;
        }
    }

  return 0;
}

static int
zgxq_eveluate (acnkern_t * eve, char pan[10][9], int bisredturn)
{
  int i, j, k, halfvalue, redvalue, blackvalue, value;
  int chesstype, targettype;

  g_count++;

  memset (eve->chessvalue, 0, sizeof (eve->chessvalue));
  memset (eve->attackpos, 0, sizeof (eve->attackpos));
  memset (eve->guardpos, 0, sizeof (eve->guardpos));
  memset (eve->flexibilitypos, 0, sizeof (eve->flexibilitypos));

  for (i = 0; i < 10; ++i)
    {
      for (j = 0; j < 9; ++j)
        {
          if (pan[i][j] != 0)
            {
              chesstype = pan[i][j];
              acnkern_get_relate (eve, pan, j, i);
              for (k = 0; k < eve->poscount; ++k)
                {
                  targettype = pan[eve->relatepos[k].y][eve->relatepos[k].x];
                  if (targettype == 0)
                    {
                      eve->flexibilitypos[i][j]++;
                    }
                  else
                    {
                      if (IS_SAME (chesstype, targettype))
                        {
                          eve->guardpos[eve->relatepos[k].
                          y][eve->relatepos[k].x]++;
                        }
                      else
                        {
                          eve->attackpos[eve->relatepos[k].
                          y][eve->relatepos[k].x]++;

                          switch (targettype)
                            {
                            case R_SHUAI:
                              if (!bisredturn)
                                {
                                  return 18888;
                                }
                              break;

                            case B_SHUAI:
                              if (bisredturn)
                                {
                                  return 18888;
                                }
                              break;

                            default:
                              eve->attackpos[eve->relatepos[k].
                              y][eve->relatepos[k].x] +=
                              (30 +
                              (eve->basevalue[targettype] -
                              eve->basevalue[chesstype]) / 10) / 10;
                              break;
                            }
                        }
                    }
                }
            }
        }
    }

  for (i = 0; i < 10; ++i)
    {
      for (j = 0; j < 9; ++j)
        {
          if (pan[i][j] != 0)
            {
              chesstype = pan[i][j];
              eve->chessvalue[i][j]++;
              eve->chessvalue[i][j] +=
              eve->flexvalue[chesstype] * eve->flexibilitypos[i][j];
              eve->chessvalue[i][j] += getbingvalue (j, i, pan);
            }
        }
    }

  for (i = 0; i < 10; ++i)
    {
      for (j = 0; j < 9; ++j)
        {
          if (pan[i][j] != 0)
            {
              chesstype = pan[i][j];
              halfvalue = eve->basevalue[chesstype] / 16;
              eve->chessvalue[i][j] += eve->basevalue[chesstype];
              if (IS_RED (chesstype))
                {
                  if (eve->attackpos[i][j])
                    {
                      if (bisredturn)
                        {
                          if (chesstype == R_SHUAI)
                            {
                              eve->chessvalue[i][j] -= 20;
                            }
                          else
                            {
                              eve->chessvalue[i][j] -= halfvalue * 2;
                              if (eve->guardpos[i][j])
                                {
                                  eve->chessvalue[i][j] += halfvalue;
                                }
                            }
                        }
                      else
                        {
                          if (chesstype == R_SHUAI)
                            {
                              return 18888;
                            }
                          eve->chessvalue[i][j] -= halfvalue * 10;
                          if (eve->guardpos[i][j])
                            {
                              eve->chessvalue[i][j] += halfvalue * 9;
                            }
                        }
                      eve->chessvalue[i][j] -= eve->attackpos[i][j];
                    }
                  else
                    {
                      if (eve->guardpos[i][j])
                        {
                          eve->chessvalue[i][j] += 5;
                        }
                    }
                }
              else
                {
                  if (eve->attackpos[i][j])
                    {
                      if (!bisredturn)
                        {
                          if (chesstype == B_SHUAI)
                            {
                              eve->chessvalue[i][j] -= 20;
                            }
                          else
                            {
                              eve->chessvalue[i][j] -= halfvalue * 2;
                              if (eve->guardpos[i][j])
                                {
                                  eve->chessvalue[i][j] += halfvalue;
                                }
                            }
                        }
                      else
                        {
                          if (chesstype == B_SHUAI)
                            {
                              return 188888;
                            }
                          eve->chessvalue[i][j] -= halfvalue * 10;
                          if (eve->guardpos[i][j])
                            {
                              eve->chessvalue[i][j] += halfvalue * 9;
                            }
                        }
                      eve->chessvalue[i][j] -= eve->attackpos[i][j];
                    }
                  else
                    {
                      if (eve->guardpos[i][j])
                        {
                          eve->chessvalue[i][j] += 5;
                        }
                    }
                }
            }
        }
    }

  redvalue = 0;
  blackvalue = 0;
  for (i = 0; i < 10; ++i)
    {
      for (j = 0; j < 9; ++j)
        {
          chesstype = pan[i][j];
          if (chesstype != 0)
            {
              if (IS_RED (chesstype))
                {
                  redvalue += eve->chessvalue[i][j];
                }
              else
                {
                  blackvalue += eve->chessvalue[i][j];
                }
            }
        }
    }

  if (bisredturn)
    {
      value = redvalue - blackvalue;
    }
  else
    {
      value = blackvalue - redvalue;
    }

  eve->eveluate_count ++;
  if (eve->callback_count
      && eve->eveluate_count >= eve->callback_count)
    {
      if (eve->callback
          && eve->callback (eve->callbackarg) == 0)
        {
          eve->quit = 1;
        }
      eve->eveluate_count = 0;
    }

  return value;
}

#define zgxq_eveluation_add_point(ev,px,py)        do {         \
    (ev)->relatepos[ev->poscount].x = (px);                     \
    (ev)->relatepos[ev->poscount].y = (py);                     \
    (ev)->poscount ++;                                          \
} while (0);
/*
static void
zgxq_eveluation_add_point (zgxq_eveluation_t * eve, int x, int y)
{
  eve->relatepos[eve->poscount].x = x;
  eve->relatepos[eve->poscount].y = y;
  eve->poscount++;
}*/

static int
zgxq_can_touch (char pan[10][9], int fx, int fy, int tx, int ty)
{
  int i, j, id, tid;

  if (fy == ty && fx == tx)
    {
      return 0;
    }

  id = pan[fy][fx];
  tid = pan[ty][tx];
  switch (id)
    {
    case R_SHUAI:
      if (tid == B_SHUAI)
        {
          if (fx != tx)
            {
              return 0;
            }
          for (i = fy - 1; i > ty; --i)
            {
              if (pan[i][fx] != 0)
                {
                  return 0;
                }
            }
        }
      else
        {
          if (ty < 7 || tx > 5 || tx < 3)
            {
              return 0;
            }
          if (abs (fy - ty) + abs (tx - fx) > 1)
            {
              return 0;
            }
        }
      break;

    case B_SHUAI:
      if (tid == R_SHUAI)
        {
          if (fx != tx)
            {
              return 0;
            }
          for (i = fy + 1; i < ty; ++i)
            {
              if (pan[i][fx] != 0)
                {
                  return 0;
                }
            }
        }
      else
        {
          if (ty > 2 || tx > 5 || tx < 3)
            {
              return 0;
            }
          if (abs (fy - ty) + abs (tx - fx) > 1)
            {
              return 0;
            }
        }
      break;

    case R_SHI:
      if (ty < 7 || tx > 5 || tx < 3)
        {
          return 0;
        }
      if (abs (fy - ty) != 1 || abs (fx - tx) != 1)
        {
          return 0;
        }
      break;

    case B_SHI:
      if (ty > 2 || tx > 5 || tx < 3)
        {
          return 0;
        }
      if (abs (fy - ty) != 1 || abs (fx - tx) != 1)
        {
          return 0;
        }
      break;

    case R_XIANG:
      if (ty < 5)
        {
          return 0;
        }
      if (abs (fx - tx) != 2 || abs (fy - ty) != 2)
        {
          return 0;
        }
      if (pan[(fy + ty) / 2][(fx + tx) / 2] != 0)
        {
          return 0;
        }
      break;

    case B_XIANG:
      if (ty > 4)
        {
          return 0;
        }
      if (abs (fx - tx) != 2 || abs (fy - ty) != 2)
        {
          return 0;
        }
      if (pan[(fy + ty) / 2][(fx + tx) / 2] != 0)
        {
          return 0;
        }
      break;

    case R_BING:
      if (ty > fy)
        {
          return 0;
        }
      if (fy > 4 && fy == ty)
        {
          return 0;
        }
      if (fy - ty + abs (tx - fx) > 1)
        {
          return 0;
        }
      break;

    case B_BING:
      if (ty < fy)
        {
          return 0;
        }
      if (fy < 5 && fy == ty)
        {
          return 0;
        }
      if (ty - fy + abs (tx - fx) > 1)
        {
          return 0;
        }
      break;

    case R_JU:
    case B_JU:
      if (fy != ty && fx != tx)
        {
          return 0;
        }
      if (fy == ty)
        {
          if (fx < tx)
            {
              for (i = fx + 1; i < tx; ++i)
                {
                  if (pan[fy][i] != 0)
                    {
                      return 0;
                    }
                }
            }
          else
            {
              for (i = tx + 1; i < fx; ++i)
                {
                  if (pan[fy][i] != 0)
                    {
                      return 0;
                    }
                }
            }
        }
      else
        {
          if (fy < ty)
            {
              for (j = fy + 1; j < ty; ++j)
                {
                  if (pan[j][fx] != 0)
                    {
                      return 0;
                    }
                }
            }
          else
            {
              for (j = ty + 1; j < fy; ++j)
                {
                  if (pan[j][fx] != 0)
                    {
                      return 0;
                    }
                }
            }
        }
      break;

    case R_MA:
    case B_MA:
      if (!((abs (tx - fx) == 1 && abs (ty - fy) == 2)
      || (abs (tx - fx) == 2 && abs (ty - fy) == 1)))
        {
          return 0;
        }
      if (tx - fx == 2)
        {
          i = fx + 1;
          j = fy;
        }
      else if (fx - tx == 2)
        {
          i = fx - 1;
          j = fy;
        }
      else if (ty - fy == 2)
        {
          i = fx;
          j = fy + 1;
        }
      else
        {
          i = fx;
          j = fy - 1;
        }
      if (pan[j][i] != 0)
        {
          return 0;
        }
      break;

    case R_PAO:
    case B_PAO:
      if (fy != ty && fx != tx)
        {
          return 0;
        }
      if (pan[ty][tx] == 0)
        {
          if (fy == ty)
            {
              if (fx < tx)
                {
                  for (i = fx + 1; i < tx; ++i)
                    {
                      if (pan[fy][i] != 0)
                        {
                          return 0;
                        }
                    }
                }
              else
                {
                  for (i = tx + 1; i < fx; ++i)
                    {
                      if (pan[fy][i] != 0)
                        {
                          return 0;
                        }
                    }
                }
            }
          else
            {
              if (fy < ty)
                {
                  for (j = fy + 1; j < ty; ++j)
                    {
                      if (pan[j][fx] != 0)
                        {
                          return 0;
                        }
                    }
                }
              else
                {
                  for (j = ty + 1; j < fy; ++j)
                    {
                      if (pan[j][fx] != 0)
                        {
                          return 0;
                        }
                    }
                }
            }
        }
      else
        {
          int count = 0;
          if (fy == ty)
            {
              if (fx < tx)
                {
                  for (i = fx + 1; i < tx; ++i)
                    {
                      if (pan[fy][i] != 0)
                        {
                          count++;
                        }
                    }
                  if (count != 1)
                    {
                      return 0;
                    }
                }
              else
                {
                  for (i = tx + 1; i < fx; ++i)
                    {
                      if (pan[fy][i] != 0)
                        {
                          count++;
                        }
                    }
                  if (count != 1)
                    {
                      return 0;
                    }
                }
            }
          else
            {
              if (fy < ty)
                {
                  for (j = fy + 1; j < ty; ++j)
                    {
                      if (pan[j][fx] != 0)
                        {
                          count++;
                        }
                    }
                  if (count != 1)
                    {
                      return 0;
                    }
                }
              else
                {
                  for (j = ty + 1; j < fy; ++j)
                    {
                      if (pan[j][fx] != 0)
                        {
                          count++;
                        }
                    }
                  if (count != 1)
                    {
                      return 0;
                    }
                }
            }
        }
      break;

    default:
      return 0;
    }

  return 1;
}

static int
acnkern_get_relate (acnkern_t * eve, char pan[10][9], int j, int i)
{
  int x, y;
  char id, flag;

  eve->poscount = 0;
  id = pan[i][j];
  switch (id)
    {
    case R_SHUAI:
    case B_SHUAI:
      for (y = 0; y < 3; ++y)
        {
          for (x = 3; x < 6; ++x)
            {
              if (zgxq_can_touch (pan, j, i, x, y))
                {
                  zgxq_eveluation_add_point (eve, x, y);
                }
            }
        }
      for (y = 7; y < 10; y++)
        {
          for (x = 3; x < 6; ++x)
            {
              if (zgxq_can_touch (pan, j, i, x, y))
                {
                  zgxq_eveluation_add_point (eve, x, y);
                }
            }
        }
      break;

    case R_SHI:
      for (y = 7; y < 10; ++y)
        {
          for (x = 3; x < 6; ++x)
            {
              if (zgxq_can_touch (pan, j, i, x, y))
                {
                  zgxq_eveluation_add_point (eve, x, y);
                }
            }
        }
      break;

    case B_SHI:
      for (y = 0; y < 3; ++y)
        {
          for (x = 3; x < 6; ++x)
            {
              if (zgxq_can_touch (pan, j, i, x, y))
                {
                  zgxq_eveluation_add_point (eve, x, y);
                }
            }
        }
      break;

    case R_XIANG:
    case B_XIANG:
      x = j + 2;
      y = i + 2;
      if (x < 9 && y < 10 && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j + 2;
      y = i - 2;
      if (x < 9 && y >= 0 && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j - 2;
      y = i + 2;
      if (x >= 0 && y < 10 && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j - 2;
      y = i - 2;
      if (x >= 0 && y >= 0 && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      break;

    case R_MA:
    case B_MA:
      x = j + 2;
      y = i + 1;
      if ((x < 9 && y < 10) && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j + 2;
      y = i - 1;
      if ((x < 9 && y >= 0) && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j - 2;
      y = i + 1;
      if ((x >= 0 && y < 10) && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j - 2;
      y = i - 1;
      if ((x >= 0 && y >= 0) && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j + 1;
      y = i + 2;
      if ((x < 9 && y < 10) && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j - 1;
      y = i + 2;
      if ((x >= 0 && y < 10) && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j + 1;
      y = i - 2;
      if ((x < 9 && y >= 0) && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      x = j - 1;
      y = i - 2;
      if ((x >= 0 && y >= 0) && zgxq_can_touch (pan, j, i, x, y))
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      break;

    case R_JU:
    case B_JU:
      x = j + 1;
      y = i;
      while (x < 9)
        {
          if (pan[y][x] == 0)
            {
              zgxq_eveluation_add_point (eve, x, y);
            }
          else
            {
              zgxq_eveluation_add_point (eve, x, y);
              break;
            }
          x++;
        }
      x = j - 1;
      y = i;
      while (x >= 0)
        {
          if (pan[y][x] == 0)
            {
              zgxq_eveluation_add_point (eve, x, y);
            }
          else
            {
              zgxq_eveluation_add_point (eve, x, y);
              break;
            }
          x--;
        }
      x = j;
      y = i + 1;
      while (y < 10)
        {
          if (pan[y][x] == 0)
            {
              zgxq_eveluation_add_point (eve, x, y);
            }
          else
            {
              zgxq_eveluation_add_point (eve, x, y);
              break;
            }
          y++;
        }
      x = j;
      y = i - 1;
      while (y >= 0)
        {
          if (pan[y][x] == 0)
            {
              zgxq_eveluation_add_point (eve, x, y);
            }
          else
            {
              zgxq_eveluation_add_point (eve, x, y);
              break;
            }
          y--;
        }
      break;

    case R_BING:
      y = i - 1;
      x = j;
      if (y >= 0)
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      if (i < 5)
        {
          y = i;
          x = j + 1;
          if (x < 9)
            {
              zgxq_eveluation_add_point (eve, x, y);
            }
          x = j - 1;
          if (x >= 0)
            {
              zgxq_eveluation_add_point (eve, x, y);
            }
        }
      break;

    case B_BING:
      y = i + 1;
      x = j;
      if (y < 10)
        {
          zgxq_eveluation_add_point (eve, x, y);
        }
      if (i > 4)
        {
          y = i;
          x = j + 1;
          if (x < 9)
            {
              zgxq_eveluation_add_point (eve, x, y);
            }
          x = j - 1;
          if (x >= 0)
            {
              zgxq_eveluation_add_point (eve, x, y);
            }
        }
      break;

    case R_PAO:
    case B_PAO:
      for (x = j + 1, y = i, flag = 0; x < 9; ++x)
        {
          if (pan[y][x] == 0)
            {
              if (!flag)
                {
                  zgxq_eveluation_add_point (eve, x, y);
                }
            }
          else
            {
              if (!flag)
                {
                  zgxq_eveluation_add_point (eve, x, y);
                  flag = 1;
                }
              else
                {
                  zgxq_eveluation_add_point (eve, x, y);
                  break;
                }
            }
        }

      for (x = j - 1, y = i, flag = 0; x >= 0; --x)
        {
          if (pan[y][x] == 0)
            {
              if (!flag)
                {
                  zgxq_eveluation_add_point (eve, x, y);
                }
            }
          else
            {
              if (!flag)
                {
                  zgxq_eveluation_add_point (eve, x, y);
                  flag = 1;
                }
              else
                {
                  zgxq_eveluation_add_point (eve, x, y);
                  break;
                }
            }
        }

      for (x = j, y = i + 1, flag = 0; y < 10; ++y)
        {
          if (pan[y][x] == 0)
            {
              if (!flag)
                {
                  zgxq_eveluation_add_point (eve, x, y);
                }
            }
          else
            {
              if (!flag)
                {
                  zgxq_eveluation_add_point (eve, x, y);
                  flag = 1;
                }
              else
                {
                  zgxq_eveluation_add_point (eve, x, y);
                  break;
                }
            }
        }

      for (x = i, y = j - 1, flag = 0; y >= 0; --y)
        {
          if (pan[y][x] == 0)
            {
              if (!flag)
                {
                  zgxq_eveluation_add_point (eve, x, y);
                }
            }
          else
            {
              if (!flag)
                {
                  zgxq_eveluation_add_point (eve, x, y);
                  flag = 1;
                }
              else
                {
                  zgxq_eveluation_add_point (eve, x, y);
                  break;
                }
            }
        }
      break;

    default:
      break;
    }

  return eve->poscount;
}

static inline unsigned long long
rand64 ()
{
  return (unsigned long long) random ()
  ^ ((unsigned long long) random ()) << 15
  ^ ((unsigned long long) random ()) << 30
  ^ ((unsigned long long) random ()) << 45
  ^ ((unsigned long long) random ()) << 60;
}

static inline unsigned int
rand32 ()
{
  return (unsigned int) random ()
  ^ ((unsigned int) random ()) << 15 ^ ((unsigned int) random ()) << 30;
}

static void
zgxq_table_cal_init (acnkern_t * table, char pan[10][9])
{
  int j, k, id;

  table->key32 = 0;
  table->key64 = 0;

  for (j = 0; j < 10; ++j)
    {
      for (k = 0; k < 9; ++k)
        {
          id = pan[j][k];
          if (id != 0)
            {
              table->key32 ^= table->hashkey32[id][j][k];
              table->key64 ^= table->hashkey64[id][j][k];
            }
        }
    }
}

static void
zgxq_table_make_move (acnkern_t * table, struct xq_move * move,
char pan[10][9])
{
  int oid, tid;

  oid = pan[move->fromy][move->fromx];
  tid = pan[move->toy][move->tox];

  table->key32 =
  table->key32 ^ table->hashkey32[oid][move->fromy][move->fromx];
  table->key64 =
  table->key64 ^ table->hashkey64[oid][move->fromy][move->fromx];
  if (tid != 0)
    {
      table->key32 =
      table->key32 ^ table->hashkey32[tid][move->toy][move->tox];
      table->key64 =
      table->key64 ^ table->hashkey64[tid][move->toy][move->tox];
    }

  table->key32 = table->key32 ^ table->hashkey32[oid][move->toy][move->tox];
  table->key64 = table->key64 ^ table->hashkey64[oid][move->toy][move->tox];
}

static void
zgxq_table_unmake_move (acnkern_t * table, struct xq_move * move, int id,
char pan[10][9])
{
  int tid;

  tid = pan[move->toy][move->tox];

  table->key32 =
  table->key32 ^ table->hashkey32[tid][move->fromy][move->fromx];
  table->key64 =
  table->key64 ^ table->hashkey64[tid][move->fromy][move->fromx];

  table->key32 = table->key32 ^ table->hashkey32[tid][move->toy][move->tox];
  table->key64 = table->key64 ^ table->hashkey64[tid][move->toy][move->tox];

  if (id != 0)
    {
      table->key32 =
      table->key32 ^ table->hashkey32[id][move->toy][move->tox];
      table->key64 =
      table->key64 ^ table->hashkey64[id][move->toy][move->tox];
    }
}

static int
zgxq_table_find (acnkern_t * table, int alpha, int beta, int depth,
int num)
{
  int x;
  zgxq_hash_t *pht;

  x = table->key32 & 0xFFFFF;
  pht = table->head[num] + x;
  //acc_debug ("find key32: %d, x: %d\n", table->key32, x);

  if (pht->depth >= depth && pht->cksum == table->key64)
    {
      switch (pht->type)
        {
        case AH_EXACT:
          return pht->eval;

        case AH_LOWER:
          if (pht->eval >= beta)
            {
              return pht->eval;
            }
          else
            {
              break;
            }

        case AH_UPPER:
          if (pht->eval <= alpha)
            {
              return pht->eval;
            }
          else
            {
              break;
            }
        }
    }

  return 66666;
}

static void
zgxq_table_store (acnkern_t * table, int type, short eval, short depth,
int num)
{
  int x;
  zgxq_hash_t *pht;

  x = table->key32 & 0xFFFFF;
  //acc_debug ("store key32: %d, x: %d\n", table->key32, x);
  pht = table->head[num] + x;

  pht->cksum = table->key64;
  pht->type = type;
  pht->eval = eval;
  pht->depth = depth;
}

static void
zgxq_table_init (acnkern_t * table)
{
  int i, j, k;

  srand ((unsigned int) time (NULL));

  for (i = 0; i < 15; ++i)
    {
      for (j = 0; j < 10; ++j)
        {
          for (k = 0; k < 9; ++k)
            {
              table->hashkey32[i][j][k] = rand32 ();
              table->hashkey64[i][j][k] = rand64 ();
              //              acc_debug ("%d,%d,%d ====>%u, %llu\n", i, j, k, table->hashkey32[i][j][k], table->hashkey64[i][j][k]);
            }
        }
    }
}
