/*
* This file is part of the acnchess package
* Copyright (C) <2010>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE fen.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2011/07/13 10:14:14 Alf*/

#include "fen.h"
#include "acnkern.h"
#include "zgxq.h"

#include <stdio.h>
#include <limits.h>
#include <string.h>
#include <ctype.h>

static char fen_zi[] = { "0KkAaBbNnRrCcPp" };
static char fen_char[CHAR_MAX] = { 0 };
static int fen_char_init = 0;

int
fen_from_string (fen_t *fen, const char *string)
{
  const char *p;
  char wb;
  int i, j;

  memset (fen->pan, 0, 10 * 9 * sizeof (char));

  if (fen_char_init == 0)
    {
      fen_char['r'] = B_JU;
      fen_char['n'] = B_MA;
      fen_char['b'] = B_XIANG;
      fen_char['a'] = B_SHI;
      fen_char['k'] = B_SHUAI;
      fen_char['c'] = B_PAO;
      fen_char['p'] = B_BING;

      fen_char['R'] = R_JU;
      fen_char['N'] = R_MA;
      fen_char['B'] = R_XIANG;
      fen_char['A'] = R_SHI;
      fen_char['K'] = R_SHUAI;
      fen_char['C'] = R_PAO;
      fen_char['P'] = R_BING;

      fen_char_init = 1;
    }

  for (i = j = 0, p = string;
       *p;
       ++ p)
    {
      if (fen_char[(int) *p])
        {
          fen->pan[i][j] = fen_char[(int) *p];
          ++ j;
        }
      else if (isdigit (*p))
        {
          j += *p - '0';
        }
      else if (*p == '/')
        {
          ++ i;
          j = 0;
        }
      else
        {
          break;
        }
    }

  sscanf (p, " %c - - %d %d",
          &wb,
          &fen->half_count,
          &fen->pair_count);

  if (wb == 'b')
    {
      fen->is_redturn = 0;
    }
  else
    {
      fen->is_redturn = 1;
    }

#ifdef _DEBUG
  output_pan (fen->pan);
#endif

  return 0;
}

int
fen_to_string (fen_t *fen, char *string, size_t size)
{
  char tmpbuf[0x100], *p;
  int i, j, cnt; 
  size_t len;

  for (p = tmpbuf, i = 0; i < 10; ++ i)
    {
      for (cnt = 0, j = 0; j < 9; ++ j)
        {
          if (fen->pan[i][j])
            {
              if (cnt)
                {
                  *p = cnt + '0';
                  ++ p;
                  cnt = 0;
                }
              *p = fen_zi[(int) fen->pan[i][j]];
              ++ p;
            }
          else 
            {
              ++ cnt;
            }
        }
      *p = '/';
      ++ p;
    }

  -- p;

  p += sprintf (p, " %c - - %d %d",
                fen->is_redturn ? 'w': 'b',
                fen->half_count,
                fen->pair_count);

  len = p - tmpbuf;
  if (size <= len)
    {
      len = size - 1;
    }
  memcpy (string, tmpbuf, len);
  string[len] = '\0';

  return 0;
}

