/*
* This file is part of the acnchess package
* Copyright (C) <2008>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE log.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2009/07/24 17:01:40 Alf*/

#include "util.h"
#include "log.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

static int (*print_func) (const char *, void *) = NULL;
static void *print_arg;

void
acclog_register (int (*cb) (const char *, void *), void *arg)
{
  print_func = cb;
  print_arg = arg;
}

void
acclog_f (const char *subsys, int loglevel, const char *format, ...)
{
  va_list ap;
  char buf[1024];
  int len;

  va_start (ap, format);

  if (print_func)
    {
      len = snprintf (buf, sizeof buf, "%s:", subsys);
      vsnprintf (buf + len, sizeof buf - len, format, ap);
      print_func (buf, print_arg);
    }
  else
    {
      printf ("%s:", subsys);
      vprintf (format, ap);
    }

  va_end (ap);
}
