/*
* This file is part of the acnchess package
* Copyright (C) <2008>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE acnchess.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2009/07/24 17:01:08 Alf*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#if defined WIN32 && defined NDEBUG
#pragma comment (linker, "/subsystem:windows")
#pragma comment (linker, "/ENTRY:mainCRTStartup")
#endif

#include "log.h"
#include "accui.h"

#ifdef ACNCHESS_ENABLE_PROFILER
#include <google/profiler.h>
#endif

#include <locale.h>
#include <libintl.h>
#include <glib/gstdio.h>
#include <signal.h>

static int init ();

int
main (int argc, char *argv[])
{
  accui_t *ui;

  init ();

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, ACNCHESS_LOCALEDIR);
  bind_textdomain_codeset (PACKAGE, "UTF-8");
  textdomain (PACKAGE);

  ui = accui_new (&argc, &argv);

#ifndef WIN32
  signal (SIGPIPE, SIG_IGN);
#endif

#ifdef ACNCHESS_ENABLE_PROFILER
  ProfilerStart ("acnchess.prof");
#endif
  accui_run (ui);
#ifdef ACNCHESS_ENABLE_PROFILER
  ProfilerStop ();
#endif

  return 0;
}

static int
init ()
{
  gchar *path, *rpath, *conts;
  gsize siz;
  GError *error = NULL;

  /*
   * Create home dir
   * */
  rpath = acc_realpath ("~/.acnchess");
  if (rpath == NULL)
    {
      acc_err (_("Memory error\n"));
      return -1;
    }

  if (g_file_test (rpath, G_FILE_TEST_IS_DIR) == FALSE)
    {
      if (g_mkdir (rpath, 0755) < 0)
        {
          acc_err (_("Create user home directory error.\n"));
          free (rpath);
          return -1;
        }
    }
  free (rpath);

  /*
   * Copy ini config file
   * */
  rpath = acc_realpath ("~/.acnchess/config");
  if (rpath == NULL)
    {
      acc_err (_("Memory error\n"));
      return -1;
    }

  if (g_file_test (rpath, G_FILE_TEST_IS_REGULAR) == FALSE)
    {
      path = g_strdup_printf ("%s/config.example", ACNCHESS_DATADIR);
      if (path == NULL)
        {
          acc_err (_("Memory error\n"));
          free (rpath);
          return -1;
        }

      if (g_file_get_contents (path, &conts, &siz, NULL))
        {
          if (g_file_set_contents (rpath, conts, siz, &error) == FALSE)
            {
              acc_err (_("set contents error: %s\n"), error->message);
              g_error_free (error);
              g_free (conts);
              free (rpath);
              return -1;
            }
          g_free (conts);
        }
    }
  free (rpath);

  rpath = acc_realpath ("~/.acnchess/server-list");
  if (rpath == NULL)
    {
      acc_err (_("Memory error\n"));
      return -1;
    }

  if (g_file_test (rpath, G_FILE_TEST_IS_REGULAR) == FALSE)
    {
      path = g_strdup_printf ("%s/server-list", ACNCHESS_DATADIR);
      if (path == NULL)
        {
          acc_err (_("Memory error\n"));
          free (rpath);
          return -1;
        }

      if (g_file_get_contents (path, &conts, &siz, NULL))
        {
          if (g_file_set_contents (rpath, conts, siz, &error) == FALSE)
            {
              acc_err (_("set contents error: %s\n"), error->message);
              g_error_free (error);
              g_free (conts);
              free (rpath);
              return -1;
            }
          g_free (conts);
        }

      g_free (path);
    }
  free (rpath);

  return 0;
}
