/*
* This file is part of the acnchess package
* Copyright (C) <2008>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE packet.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2009/08/12 13:12:56 Alf*/

#include "log.h"
#include "packet.h"

#include <stdlib.h>
#include <string.h>
#ifndef WIN32
#include <arpa/inet.h>
#else
#include <winsock2.h>
#endif

int
packet_encode (packet_t * packet, char *buffer, unsigned short cap)
{
  packet_elem *elem;
  int remain;

  remain = cap;

  if (remain >= 2 * sizeof (unsigned short))
    {
      *(unsigned short *) buffer = htons (packet->type);
      buffer += sizeof (unsigned short);
      remain -= sizeof (unsigned short);
      *(unsigned short *) buffer = htons (packet->version);
      buffer += sizeof (unsigned short);
      remain -= sizeof (unsigned short);
    }
  else
    {
      return -1;
    }

  for (elem = packet->elem; elem != NULL; elem = elem->next)
    {
      if (remain >=
          2 * sizeof (unsigned short) + sizeof (unsigned short) +
          elem->length)
        {
          *(unsigned short *) buffer = htons (elem->type);
          buffer += sizeof (unsigned short);
          remain -= sizeof (unsigned short);
          *(unsigned short *) buffer = htons (elem->length);
          buffer += sizeof (unsigned short);
          remain -= sizeof (unsigned short);
          memcpy (buffer, elem->data, elem->length);
          remain -= elem->length;
        }
      else
        {
          return -1;
        }
    }

  acc_debug ("encode packet to %d buffer\n", cap - remain);
  return cap - remain;
}

int
packet_decode (const char *buffer, unsigned short length, packet_t * packet)
{
  packet_elem *elem;
  int remain;

  acc_debug ("decode %d buffer to packet\n", length);
  remain = length;

  if (remain >= 2 * sizeof (unsigned short))
    {
      packet->type = ntohs (*(unsigned short *) buffer);
      buffer += sizeof (unsigned short);
      remain -= sizeof (unsigned short);
      packet->version = ntohs (*(unsigned short *) buffer);
      buffer += sizeof (unsigned short);
      remain -= sizeof (unsigned short);
    }
  else
    {
      return -1;
    }

  for (packet->elem = NULL; remain > 0;)
    {
      elem = calloc (1, sizeof (packet_elem));
      if (elem == NULL)
        {
          acc_err (_("Memory error\n"));
          packet_free (packet);
          return -1;
        }

      if (remain >= 2 * sizeof (unsigned short))
        {
          elem->type = ntohs (*(unsigned short *) buffer);
          buffer += sizeof (unsigned short);
          remain -= sizeof (unsigned short);
          elem->length = ntohs (*(unsigned short *) buffer);
          buffer += sizeof (unsigned short);
          remain -= sizeof (unsigned short);
        }
      else
        {
          acc_err (_("decode error\n"));
          packet_free (packet);
          return -1;
        }

      if (remain >= elem->length && elem->length <= sizeof elem->data)
        {
          memcpy (elem->data, buffer, elem->length);
          buffer += elem->length;
          remain -= elem->length;
        }
      else
        {
          acc_err (_("decode error\n"));
          packet_free (packet);
          return -1;
        }

      if (packet->elem == NULL)
        {
          packet->elem = elem;
        }
      else
        {
          elem->next = packet->elem;
          packet->elem = elem;
        }
    }

  return 0;
}

void
packet_free (packet_t * packet)
{
  packet_elem *elem;

  for (elem = packet->elem; elem != NULL; elem = packet->elem)
    {
      packet->elem = elem->next;
      free (elem);
    }
}

packet_elem *
packet_elem_new (unsigned short type, int len, const char *data)
{
  packet_elem *elem;

  if (len > ELEM_MAX)
    {
      return NULL;
    }

  elem = malloc (sizeof (packet_elem));
  if (elem == NULL)
    {
      acc_err (_("Memory error\n"));
      return NULL;
    }

  elem->type = type;
  elem->length = 0;
  if (len > 0 || data != NULL)
    {
      elem->length = len > 0 ? len : strlen (data);
      memcpy (elem->data, data, elem->length);
    }
  elem->next = NULL;

  return elem;
}
