/*
* This file is part of the acnchess package
* Copyright (C) <2008>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE tport.h
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2009/07/24 17:14:45 Alf*/

#ifndef _ACC_TPORT_H_
#define _ACC_TPORT_H_

#ifdef WIN32
#include <winsock2.h>

#define socket_t SOCKET

#define read(x,y,z)     recv ((x), (y), (z), 0)
#define write(x,y,z)    send ((x), (y), (z), 0)
#define close           closesocket

#define EINPROGRESS	    WSAEINPROGRESS
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#define socket_t int
#endif

#define TPORTLOCALIP(t)       ((t)->loc->sin_addr.s_addr)

#define TPORTLOCALPORT(t)       ((t)->loc->sin_port)

enum
{ UDP, TCP, TLS };

typedef struct
{
  socket_t fd;

  int type;
  struct sockaddr_in loc[1];
  struct sockaddr_in rmt[1];
} tport_t;

tport_t *tport_new (int, const char *, unsigned short);

tport_t *tport_new_with_fd (int, int);

int tport_connect (tport_t *, unsigned long, unsigned short, int);

int tport_write (tport_t *, const char *, int, const struct sockaddr_in *);

int tport_read (tport_t *, char *, int, struct sockaddr_in *, int);

void tport_delete (tport_t *);


/*
 * Useful external util to set socket
 * */
int socket_block (int);

int socket_nonblock (int);

#endif
