/*
* This file is part of the acnchess package
* Copyright (C) <2010>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE ucci.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2011/07/13 10:09:44 Alf*/

#include "ucci.h"
#include "log.h"

#include <stdio.h>
#include <string.h>

const char *ucci_cmd_phrase[] = {
    "ucci",
    "id",
    "option",
    "ucciok",
    "isready",
    "readyok",
    "setoption",
    "position",
    "banmoves",
    "go",
    "info",
    "ponderhit",
    "stop",
    "bestmove",
    "nobestmove",
    "probe",
    "pophash",
    "quit",
    "bye",
};

#define ucci_cmd_phrase_count (sizeof (ucci_cmd_phrase) / sizeof (ucci_cmd_phrase[0]))

int 
ucci_cmd_to_string (ucci_cmd *cmd, char *out, size_t out_len)
{
  return 0;
}

extern const char *ucci_cmd_phrase[];
int 
ucci_cmd_from_string (ucci_cmd *cmd, const char *in)
{
  char tmp[0x2000], *p, *cmdphrase;
  size_t i;
  int len;

  len = snprintf (tmp, sizeof tmp, "%s", in);
  if (tmp[len - 1] == '\n')
    {
      tmp[len - 1] = '\0';
    }

  p = strtok (tmp, " ");
  if (p == NULL)
    {
      cmdphrase = tmp;
    }
  else
    {
      cmdphrase = p;
    }

  for (i = 0; i < ucci_cmd_phrase_count; ++ i)
    {
      if (strcmp (ucci_cmd_phrase[i], cmdphrase) == 0)
        {
          break;
        }
    }

  if (i == ucci_cmd_phrase_count)
    {
      return -1;
    }

  cmd->cmd = i;

  for (cmd->arg_count = 0;
       (p = strtok (NULL, " "));
       ++ cmd->arg_count)
    {
      snprintf (cmd->args[cmd->arg_count], sizeof (cmd->args[cmd->arg_count]), 
                "%s", p);
    }

#ifdef _DEBUG

  fprintf (stderr, "%s (", ucci_cmd_phrase[cmd->cmd]);
  for (i = 0; i < cmd->arg_count; ++ i)
    {
      fprintf (stderr, "%s,", cmd->args[i]);
    }
  fprintf (stderr, ")\n");

#endif

  return 0;
}
