﻿/*
* This file is part of the acnchess package
* Copyright (C) <2010>  <Alf>
*
* Contact: Alf <naihe2010@126.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/
/* @CFILE zgxq.c
*
*  Author: Alf <naihe2010@126.com>
*/
/* @date Created: 2011/07/14 15:08:45 Alf*/

#include "zgxq.h"

#include <stdio.h>
#include <stdlib.h>

char default_pan[10][9] =
{
  {B_JU, B_MA, B_XIANG, B_SHI, B_SHUAI, B_SHI, B_XIANG, B_MA, B_JU,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, B_PAO, 0, 0, 0, 0, 0, B_PAO, 0,},
  {B_BING, 0, B_BING, 0, B_BING, 0, B_BING, 0, B_BING,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {R_BING, 0, R_BING, 0, R_BING, 0, R_BING, 0, R_BING,},
  {0, R_PAO, 0, 0, 0, 0, 0, R_PAO, 0,},
  {0, 0, 0, 0, 0, 0, 0, 0, 0,},
  {R_JU, R_MA, R_XIANG, R_SHI, R_SHUAI, R_SHI, R_XIANG, R_MA, R_JU,},
};

char *number_str[] = {
    "0",
    "1",
    "2",
    "3",
    "4",
    "5",
    "6",
    "7",
    "8",
    "9",
};

char *number_b_str[] = {
    "０",
    "１",
    "２",
    "３",
    "４",
    "５",
    "６",
    "７",
    "８",
    "９",
};

char *zi_chinese[] = {
    "",
    "帅",
    "将",
    "士",
    "士",
    "相",
    "象",
    "马",
    "马",
    "车",
    "车",
    "炮",
    "炮",
    "兵",
    "卒",
};

char *number_chinese_str[] = {
    "零",
    "一",
    "二",
    "三",
    "四",
    "五",
    "六",
    "七",
    "八",
    "九",
};

int
zgxq_is_over (char pan[10][9])
{
  int i, j, rl, bl;

  rl = 0;
  bl = 0;

  for (i = 7; i < 10; ++i)
    {
      for (j = 3; j < 6; ++j)
        {
          if (pan[i][j] == R_SHUAI)
            {
              rl = 1;
              break;
            }
          if (pan[i][j] == B_SHUAI)
            {
              bl = 1;
              break;
            }
        }
    }

  for (i = 0; i < 3; ++i)
    {
      for (j = 3; j < 6; ++j)
        {
          if (pan[i][j] == B_SHUAI)
            {
              bl = 1;
              break;
            }
          if (pan[i][j] == R_SHUAI)
            {
              rl = 1;
              break;
            }
        }
    }

  return rl - bl;
}

int
zgxq_move_is_valid (char pan[10][9], int fx, int fy, int tx, int ty)
{
  int i, j, oid, tid;

  if (fx == tx && fy == ty)
    {
      return 0;
    }

  oid = pan[fy][fx];
  tid = pan[ty][tx];

  if (IS_SAME (oid, tid))
    {
      return 0;
    }

  switch (oid)
    {
    case R_SHUAI:
      if (tid == B_SHUAI)
        {
          if (fx != tx)
            {
              return 0;
            }
          for (i = fy - 1; i > ty; --i)
            {
              if (pan[i][fx] != 0)
                {
                  return 0;
                }
            }
        }
      else
        {
          if (ty < 7 || tx < 3 || tx > 5)
            {
              return 0;
            }
          if (abs (fy - ty) + abs (tx - fx) > 1)
            {
              return 0;
            }
        }
      break;

    case B_SHUAI:
      if (tid == R_SHUAI)
        {
          if (fx != tx)
            {
              return 0;
            }
          for (i = fy + 1; i < ty; ++i)
            {
              if (pan[i][fx] != 0)
                {
                  return 0;
                }
            }
        }
      else
        {
          if (ty > 2 || tx < 3 || tx > 5)
            {
              return 0;
            }
          if (abs (fy - ty) + abs (tx - fx) > 1)
            {
              return 0;
            }
        }
      break;

    case R_SHI:
      if (ty < 7 || tx < 3 || tx > 5)
        {
          return 0;
        }
      if (abs (fy - ty) != 1 || abs (tx - fx) != 1)
        {
          return 0;
        }
      break;

    case B_SHI:
      if (ty > 2 || tx < 3 || tx > 5)
        {
          return 0;
        }
      if (abs (fy - ty) != 1 || abs (tx - fx) != 1)
        {
          return 0;
        }
      break;

    case R_XIANG:
      if (ty < 5)
        {
          return 0;
        }
      if (abs (ty - fy) != 2 || abs (tx - fx) != 2)
        {
          return 0;
        }
      if (pan[(fy + ty) / 2][(fx + tx) / 2] != 0)
        {
          return 0;
        }
      break;

    case B_XIANG:
      if (ty > 4)
        {
          return 0;
        }
      if (abs (ty - fy) != 2 || abs (tx - fx) != 2)
        {
          return 0;
        }
      if (pan[(fy + ty) / 2][(fx + tx) / 2] != 0)
        {
          return 0;
        }
      break;

    case R_BING:
      if (ty > fy)
        {
          return 0;
        }
      if (fy > 4 && fy == ty)
        {
          return 0;
        }
      if (fy - ty + abs (tx - fx) > 1)
        {
          return 0;
        }
      break;

    case B_BING:
      if (ty < fy)
        {
          return 0;
        }
      if (fy < 5 && fy == ty)
        {
          return 0;
        }
      if (ty - fy + abs (tx - fx) > 1)
        {
          return 0;
        }
      break;

    case R_JU:
    case B_JU:
      if (fy != ty && fx != tx)
        {
          return 0;
        }
      if (fy == ty)
        {
          if (fx < tx)
            {
              for (i = fx + 1; i < tx; ++i)
                {
                  if (pan[fy][i] != 0)
                    {
                      return 0;
                    }
                }
            }
          else
            {
              for (i = tx + 1; i < fx; ++i)
                {
                  if (pan[fy][i] != 0)
                    {
                      return 0;
                    }
                }
            }
        }
      else
        {
          if (fy < ty)
            {
              for (i = fy + 1; i < ty; ++i)
                {
                  if (pan[i][fx] != 0)
                    {
                      return 0;
                    }
                }
            }
          else
            {
              for (i = ty + 1; i < fy; ++i)
                {
                  if (pan[i][tx] != 0)
                    {
                      return 0;
                    }
                }
            }
        }
      break;

    case R_MA:
    case B_MA:
      if (!(abs (tx - fx) == 1 && abs (ty - fy) == 2)
      && !(abs (tx - fx) == 2 && abs (ty - fy) == 1))
        {
          return 0;
        }
      if (tx - fx == 2)
        {
          i = fx + 1;
          j = fy;
        }
      else if (fx - tx == 2)
        {
          i = tx + 1;
          j = fy;
        }
      else if (ty - fy == 2)
        {
          i = fx;
          j = fy + 1;
        }
      else /* if (fy - ty == 2) */
        {
          i = fx;
          j = ty + 1;
        }
      if (pan[j][i] != 0)
        {
          return 0;
        }
      break;

    case R_PAO:
    case B_PAO:
      if (fx != tx && fy != ty)
        {
          return 0;
        }
      if (pan[ty][tx] == 0)
        {
          if (fy == ty)
            {
              if (fx < tx)
                {
                  for (i = fx + 1; i < tx; ++i)
                    {
                      if (pan[fy][i] != 0)
                        {
                          return 0;
                        }
                    }
                }
              else
                {
                  for (i = tx + 1; i < fx; ++i)
                    {
                      if (pan[fy][i] != 0)
                        {
                          return 0;
                        }
                    }
                }
            }
          else
            {
              if (fy < ty)
                {
                  for (j = fy + 1; j < ty; ++j)
                    {
                      if (pan[j][fx] != 0)
                        {
                          return 0;
                        }
                    }
                }
              else
                {
                  for (j = ty + 1; j < fy; ++j)
                    {
                      if (pan[j][fx] != 0)
                        {
                          return 0;
                        }
                    }
                }
            }
        }
      else
        {
          int count = 0;
          if (fy == ty)
            {
              if (fx < tx)
                {
                  for (i = fx + 1; i < tx; ++i)
                    {
                      if (pan[fy][i] != 0)
                        {
                          count++;
                        }
                    }
                  if (count != 1)
                    {
                      return 0;
                    }
                }
              else
                {
                  for (i = tx + 1; i < fx; ++i)
                    {
                      if (pan[fy][i] != 0)
                        {
                          count++;
                        }
                    }
                  if (count != 1)
                    {
                      return 0;
                    }
                }
            }
          else
            {
              if (fy < ty)
                {
                  for (j = fy + 1; j < ty; ++j)
                    {
                      if (pan[j][fx] != 0)
                        {
                          count++;
                        }
                    }
                  if (count != 1)
                    {
                      return 0;
                    }
                }
              else
                {
                  for (j = ty + 1; j < fy; ++j)
                    {
                      if (pan[j][fx] != 0)
                        {
                          count++;
                        }
                    }
                  if (count != 1)
                    {
                      return 0;
                    }
                }
            }
        }
      break;
    }

  return 1;
}

void
output_pan (char pan[10][9])
{
  int i, j;

  for (i = 0; i < 10; ++ i)
    {
      printf ("\t\t");
      for (j = 0; j < 9; ++ j)
        {
          if (pan[i][j])
            {
              printf ("[%s]", zi_chinese[(int)pan[i][j]]);
            }
          else
            {
              printf ("[  ]");
            }
        }
      printf ("\n");
    }
}
